# Sitemap status

Logs the http status for each sitemap location in the status report
and adds an entry to the Admin Toolbar.
Optionally checks for errors or specific elements in the DOM.

Use case: while doing a deployment (production or pull request based environment),
be able to check the status of each page from the sitemap based on a production database.
As unit/E2E tests might contain testing gaps and/or possibly not catch "content based"
issues (like redirects/aliases), this can be used as an affordable smoke test.

| Status check            | Toolbar result | Executed by default |
| :---------------------: | :------------: | :-----------------: |
| 5xx http status         | ❌             | Yes                 |
| DOM error               | ❌             | Optional            |
| DOM missing element     | ❌             | Optional            |
| 4xx http status         | ⚠️             | Yes                 |
| 3xx, 2xx http status    | ✅             | Yes                 |
| Unreachable             | ℹ              | Yes                 |
| Not checked yet         | ❓             | n/a                 |

## Dependencies

- [Simple XML Sitemap](https://www.drupal.org/project/simple_sitemap)
- `curl` and `libxml` PHP extensions

## Configuration

Can be used without any configuration via drush.

Optional configuration for UI or cron based checks:
Configuration > Development > Sitemap status
(/admin/config/development/sitemap-status).

Drush can be used on build e.g. within a post-rollout task.
Example: generate the sitemap then run the check.

```
- run:
    name: drush ssg
    command: drush simple-sitemap:generate
    service: cli
- run:
    name: drush ssc
    command: drush sitemap-status:check
    service: cli
```

Example configuration for error classes

```
messages--error
messages--warning
form-item--error-message
```

Example configuration for elements

```
title
h1
header
```

## Usage

`drush sitemap-status:check` (alias: drush ssc)

Options examples:
- adds basic auth for PR / staging environments
- adds a path prefix (e.g. to isolate other 301 redirects)
- use async check with concurrent requests set to 15 (defaults to 10)
- check for errors in the DOM (uses configuration or fallback to `messages--error`)
- check for elements in the DOM (uses configuration or fallback to `h1`)
- skip the check on excluded urls (e.g. not in the production build)
- stop the check on the first error met

```
drush ssc \
--basic-auth=username:password \
--path-prefix=europe \
--async=true \
--concurrent-requests=15 \
--errors=true \
--elements=true \
--excluded-urls=https://nginx-production.mysite.org,https://mysite.org \
--stop-on-error=true
```

## Credits

Thank you [@dan2k3k4](https://www.drupal.org/u/dan2k3k4) for the original idea :)

## Roadmap

- Define arbitrary url's, including absolute ones with pattern depending on the environment ([staging|prod].subdomain.org)
- Optionally stop on error (batch, from the UI)
- Limit the locations to test, by url pattern to cover a sample of content
- Randomize the locations to test
- Improve the integration with the Toolbar, make it optional
- Optional follow redirect
- Expose the sitemap status as json for external polling
- Improve options validation
- Improve caching invalidation of the Toolbar
- ~~Optionally preserve cache after a cache rebuild~~
- ~~Limit the locations to test, by amount~~
- ~~Only http status is checked, add error or element (e.g. title) check~~
- ~~Reduce check time, optionally use curl_multi~~
- ~~Trigger the status check from the ui~~
- ~~Optional cron based check~~
- ~~Make use of a specific cache backend~~
