<?php

namespace Drupal\sitemap_status\Plugin\QueueWorker;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\sitemap_status\SitemapStatusInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Fetches the http status for a group of locations.
 *
 * @QueueWorker(
 *   id = "locations_group",
 *   title = @Translation("Fetches the http status for a group of locations."),
 * )
 */
class LocationsGroupQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\sitemap_status\SitemapStatusInterface definition.
   *
   * @var \Drupal\sitemap_status\SitemapStatusInterface
   */
  protected $sitemapStatus;

  /**
   * ReleaseFilesDuplicateQueueWorker constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $sitemap_status
   */
  public function __construct(SitemapStatusInterface $sitemap_status) {
    $this->sitemapStatus = $sitemap_status;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('sitemap_status')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $result = [];
    $processResult = $this->sitemapStatus->fetchLocationStatusMultiple((array) $data);
    if (is_array($processResult)) {
      $result = $processResult;
    }
    return $result;
  }

}
